{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- /No description available in the introspection data./

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.Cairo.Structs.ScaledFont
    ( 

-- * Exported types
    ScaledFont(..)                          ,


 -- * Methods

#if defined(ENABLE_OVERLOADING)
    ResolveScaledFontMethod                 ,
#endif



    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)

#else

#endif

-- | Memory-managed wrapper type.
newtype ScaledFont = ScaledFont (SP.ManagedPtr ScaledFont)
    deriving (Eq)

instance SP.ManagedPtrNewtype ScaledFont where
    toManagedPtr (ScaledFont p) = p

foreign import ccall "cairo_gobject_scaled_font_get_type" c_cairo_gobject_scaled_font_get_type :: 
    IO GType

type instance O.ParentTypes ScaledFont = '[]
instance O.HasParentTypes ScaledFont

instance B.Types.TypedObject ScaledFont where
    glibType = c_cairo_gobject_scaled_font_get_type

instance B.Types.GBoxed ScaledFont

-- | Convert t'ScaledFont' to and from 'Data.GI.Base.GValue.GValue'. See 'Data.GI.Base.GValue.toGValue' and 'Data.GI.Base.GValue.fromGValue'.
instance B.GValue.IsGValue (Maybe ScaledFont) where
    gvalueGType_ = c_cairo_gobject_scaled_font_get_type
    gvalueSet_ gv P.Nothing = B.GValue.set_boxed gv (FP.nullPtr :: FP.Ptr ScaledFont)
    gvalueSet_ gv (P.Just obj) = B.ManagedPtr.withManagedPtr obj (B.GValue.set_boxed gv)
    gvalueGet_ gv = do
        ptr <- B.GValue.get_boxed gv :: IO (Ptr ScaledFont)
        if ptr /= FP.nullPtr
        then P.Just <$> B.ManagedPtr.newBoxed ScaledFont ptr
        else return P.Nothing
        
    


#if defined(ENABLE_OVERLOADING)
instance O.HasAttributeList ScaledFont
type instance O.AttributeList ScaledFont = ScaledFontAttributeList
type ScaledFontAttributeList = ('[ ] :: [(Symbol, DK.Type)])
#endif

#if defined(ENABLE_OVERLOADING)
type family ResolveScaledFontMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveScaledFontMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveScaledFontMethod t ScaledFont, O.OverloadedMethod info ScaledFont p) => OL.IsLabel t (ScaledFont -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveScaledFontMethod t ScaledFont, O.OverloadedMethod info ScaledFont p, R.HasField t ScaledFont p) => R.HasField t ScaledFont p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveScaledFontMethod t ScaledFont, O.OverloadedMethodInfo info ScaledFont) => OL.IsLabel t (O.MethodProxy info ScaledFont) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif


